'''
CS 1, Lecture 21

Implementation of a variety of sorting algorithms, including
.  Selection Sort,
.  Insertion Sort
.  Merge Sort
Details will be filled in during lecture.  

The main code gives tests for these functions.


Insertion sort pseudo code:

for each index i in the list, starting at 1 do
    Save the value stored at location i in variable val
    Initialize j at location i-1
    while j is non-negative and the location to insert x has not been found do
        Shift the value at location j up to location j+1
        Decrement j
    Insert the value stored in val in location j+1
'''

def ins_sort( L ):
    '''
    The Insertion Sort algorithm:
    Complexity: O(N^2)
    
    >>> ins_sort([5,4,3,2,1])
    [1, 2, 3, 4, 5]
    >>> ins_sort([1])
    [1]
    >>> ins_sort([1,2,3,4])
    [1, 2, 3, 4]
    >>> ins_sort([])
    []
    
    '''
    #  Fill in here
    for i in range(1,len(L)):
        val = L[i]
        j = i-1
        while j>=0 and L[j]>val:
            L[j+1] = L[j]
            j -= 1
        L[j+1] = val
    return (L)
        
def sel_sort( v ):
    '''
    Sort based on the O(N^2) selection sort algorithm.  The ideas is
    discussed in the text book.  Students are not responsible for
    knowing this algorithm.
    '''
    for i in range(0, len(v)-1):
        k = i
        for j in range(i+1,len(v)):
            if v[j] < v[k]:
                k = j
        v[i],v[k] = v[k],v[i]




def merge(L1,L2):
    '''
    Merge the contents of two lists, each of which is already sorted
    into a single sorted list.
    '''
    i1 = 0
    i2 = 0
    L = []
    while i1 < len(L1) and i2 < len(L2):
        if L1[i1] < L2[i2]:
            L.append ( L1[i1] )
            i1 += 1
        else:
            L.append( L2[i2] )
            i2 += 1
    L.extend( L1[i1:] )
    L.extend( L2[i2:] )
    return(L)
            

def merge_sort(L):
    '''
    Implementation of the merge sort algorithm.
    Complexity: O(n logn)
    
    >>> merge_sort([5,4,3,2,1])
    [1, 2, 3, 4, 5]
    >>> merge_sort([1])
    [1]
    >>> merge_sort([1,2,3,4])
    [1, 2, 3, 4]
    >>> merge_sort([])
    []
    
    '''
    if len(L) <= 1:
        return (L)

    M = []
    for val in L:
        M.append( [val] )
    
    while len(M)>1:
        M2 = []
        for i in range(0,len(M)-1,2):
            M2.append( merge( M[i], M[i+1] ))
        if len(M)%2 == 1:
            M2.append(M[-1])
        M = M2
    return M[0]






if __name__ == "__main__":
    '''
    v =  [ 10, 5, 3, 2, -4 ]
    ins_sort(v)
    print(v)
    v = []
    ins_sort(v)
    print(v)
    v = [ 5, 6, 7, 6, 5, 5]
    ins_sort(v)
    print(v)
    '''

    '''
    v1 = [ 26, 35, 145]
    v0 = [ 0, 0, 9, 9, 9.4, 9.6, 15, 21 ]
    print(v0)
    print(v1)
    print(merge(v0,v1))
    '''

    #v = [ 9.1, 17.5, 9.8, 6.3, 12.4, 1.7 ]
    #merge_sort(v)
    #print(v)
    pass   

